# Copyright 2007, 2008, 2010 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.

"""Pylons environment configuration"""

import os
import logging
from urlparse import urlparse
from urllib import unquote

from mako.lookup import TemplateLookup
from pylons.error import handle_mako_error

from pylons.configuration import PylonsConfig

import mpulsweb.lib.helpers
from mpulsweb.config.routing import make_map
from mpulsweb.config.importer import import_overridable_module


log = logging.getLogger(__name__)


def load_environment(global_conf, app_conf):
    """Configure the Pylons environment via the ``pylons.config`` object"""
    config = PylonsConfig()

    def get(item):
        """Lookup item in app_conf and global_conf.  Return the first one found.
        If item is found in neither, None is returned.
        """
        if item in app_conf:
            return app_conf[item]
        return global_conf.get(item)

    # Pylons paths
    root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

    # Templates
    template_urls = []
    custom_templates = get('mpuls.app.path.templates')
    if custom_templates:
        log.info('Custom "template" directory: %s' % custom_templates)
        template_urls.append(custom_templates)
    template_urls.append(os.path.join(root, 'templates'))

    # Static content (images, styles, formed...)
    static_urls = []
    custom_static = get('mpuls.app.path.public')
    if custom_static:
        log.info('Custom "public" directory: %s' % custom_static)
        static_urls.append(get('mpuls.app.path.public'))
    static_urls.append(os.path.join(root, 'public'))

    # Controllers
    controller_url = os.path.join(root, 'controllers')
    if get('mpuls.app.path.controllers'):
        controller_url = get('mpuls.app.path.controllers')
        log.info('Custom "controller" directory: %s' % controller_url)

    # Internationalisation 
    i18n_url = os.path.join(root, 'i18n')
    if get('mpuls.app.path.i18n'):
        i18n_url = get('mpuls.app.path.i18n')
        log.info('Custom "i18n" directory: %s' % i18n_url)

    paths = dict(root=root,
                 controllers=controller_url,
                 static_files=static_urls,
                 templates=template_urls,
                 i18n=i18n_url)

    # Initialize config with the basic options
    config.init_app(global_conf, app_conf, package='mpulsweb',
                    paths=paths)

    # load beaker secret from file, if so configured
    dereference_file_url(config, "beaker.session.secret")

    routing = import_overridable_module("config.routing", config)
    config['routes.map'] = routing.make_map(config)

    app_globals_module = import_overridable_module("lib.app_globals", config)
    config['pylons.app_globals'] = app_globals_module.Globals(config)
    config['pylons.h'] = mpulsweb.lib.helpers
    config['pylons.strict_tmpl_context'] = False

    # Create the Mako TemplateLookup, with the default auto-escaping
    module_directory = config.get('mpuls.app.path.template_cache',
                                  os.path.join(config['pylons.cache_dir'],
                                               'templates'))
    config['pylons.app_globals'].mako_lookup = \
        TemplateLookup(directories=paths['templates'],
                       error_handler=handle_mako_error,
                       module_directory=module_directory,
                       input_encoding='utf-8', default_filters=['escape'],
                       imports=['from webhelpers.html import escape'])

    return config


def dereference_file_url(conf, key, strip=False):
    """Dereference file:-URLs in the value of key in the dictionary conf.

    This means that if the value is a string that looks like a file-URL,
    i.e. a string which when parsed with urlparse turns out to be URL
    using the 'file' URL scheme, the rest of the URL must be the file
    URL of a file on the local file system. The contents of this file
    are read and put into the conf dictionary as the new value of for
    the key.

    If the string is not a file URL or the dictionary does not have a
    value for the key, nothing happens.

    If the string is a file URL but does not point to a file on
    localhost or the file doesn't exist or cannot be read, this
    functions will throw exceptions.

    The paramter stip can be set to True to remove whitespace at the beginning
    and end of the file.
    """
    value = conf.get(key)
    if value is None:
        return

    parsed = urlparse(value)
    if parsed.scheme != "file":
        return

    if parsed.netloc not in ("", "localhost"):
        raise ValueError("Value for %r has non-local host: %r" % (key, value))

    for attr in ("params", "query", "fragment"):
        if getattr(parsed, attr) != "":
            raise ValueError("Value for %r has non-empty %s: %r"
                             % (key, attr, value))

    with open(unquote(parsed.path)) as f:
        content = f.read()
        if strip:
            content = content.strip()
        conf[key] = content
