#!/bin/sh

set -e

usage()
{
cat << EOF
usage: $0 options

This script will import a sqlite database file into a given database.

NOTE: Currently there is a bug in the importer so that at least one table
specified with the "-t" option must be provided.

OPTIONS:
   -h      Show this message
   -c      Import command to use. Default: import-sqlite.py
   -d      Name of the database where the data will be imported.
   -w      Located of the working directory. Default to current dir.
   -f      Import file.
   -p      Port of the database cluster to import into.
   -t      Comma-separated list of tables which depend on the master_tbl which
           will be about to be exported
   -l      Copy logbook
   -j      Update fkz tbl with job_percentage
   -n      Deactivate set permissions from nmuserfkz table.
   -v      Verbose (unused)
EOF
}

# DEFAULTS
VERBOSE=0
JOBPERCENTAGE=""
IMPORTCOMMAND=import-sqlite.py
DATEBASE=
DATE=`date +"%Y+%m+%d"`
DBPORT=
RELATIONS=""
TEMPLATE=
OWNER=1
WORKING_DIR=`pwd`
COPY_LOGBOOK=
NMUSERFKZ=1

# Parse Options
while getopts "hvljnc:d:f:p:t:u:w:" OPTION
do
        case $OPTION in
            h)
                usage
                exit 1
                ;;
            c)
                IMPORTCOMMAND=$OPTARG
                ;;
            d)
                DATABASE=${OPTARG}
                ;;
            f)
                TEMPLATE=$OPTARG
                ;;
            p)
                DBPORT=$OPTARG
                ;;
            t)
                RELATIONS=$OPTARG
                ;;
            u)
                OWNER=$OPTARG
                ;;
            v)
                VERBOSE=1
                ;;
            j)
                JOBPERCENTAGE=1
                ;;
	    n)
		NMUSERFKZ=""
		;;
            w)
                WORKING_DIR=$OPTARG
                ;;
            l)
                COPY_LOGBOOK=-l
                ;;
            ?)
                # Will be set if the optstring begins with a ":". In this case
                # all options not defined in the optstring will OPTION set to
                # "?". Otherwise the shell reports an error on the unknown
                # option.
                usage
                exit
                ;;
        esac
done

# Check Options
if [ -z $DATABASE ]
then
    echo "Error! DB for import required!";
    usage;
    exit 1;
fi

if [ -z $DBPORT ]
then
    echo "Error! Port of the database to import is required!";
    usage;
    exit 1;
fi

if [ -z $TEMPLATE ]
then
    echo "Error! Templatefile for import is required!";
    usage;
    exit 1;
fi

# Finally excecute program
echo -- START Import ${DATABASE} ${DATE} --  
echo "Will import ${TEMPLATE} into ${DATABASE} on ${DBPORT}"
DIRECTORY=`dirname ${TEMPLATE}`
IMPORTFILE=${DIRECTORY}/`basename ${TEMPLATE} .bz2`
bunzip2 -kf ${TEMPLATE}
chmod 666 ${IMPORTFILE}
sudo -u postgres psql -p ${DBPORT} ${DATABASE} -c "TRUNCATE master_tbl CASCADE"
OLDIFS="$IFS"
IFS=","
for rel in ${RELATIONS}; do sudo -u postgres psql -p ${DBPORT} ${DATABASE} -c "TRUNCATE $rel CASCADE" ; done
IFS=$OLDIFS
sudo -u postgres ${WORKING_DIR}/${IMPORTCOMMAND} -i ${IMPORTFILE} -o ${DATABASE} -p ${DBPORT} --relations=${RELATIONS} ${COPY_LOGBOOK}

if [ ${JOBPERCENTAGE} ]
then
       echo "Updating the job percentage values"
       sudo -u postgres ${WORKING_DIR}/helpers/jp/import_jp.py -i ${IMPORTFILE} -o ${DATABASE} -p ${DBPORT}
fi

# Set owner to given userid
sudo -u postgres psql -p ${DBPORT} ${DATABASE} -c "UPDATE master_tbl SET bearbeiter_id = ${OWNER}"

if [ ${NMUSERFKZ} ]
then
	# Set permissions based on nm_user_fkz table
	echo "Setting fkz permission for users..."
	sudo -u postgres psql -p ${DBPORT} ${DATABASE} -c "DELETE FROM nm_benutzer_master_tbl"
	sudo -u postgres psql -p ${DBPORT} ${DATABASE} -c "INSERT INTO nm_benutzer_master_tbl (benutzer_id, master_id) select b.gid, m.id from nm_user_fkz nm JOIN fkz_tbl f ON f.id = nm.fkz_id JOIN ka_benutzer_tbl b ON b.id = nm.user_id JOIN master_tbl m ON m.fkz = f.fkz"
fi

rm ${IMPORTFILE}
echo -- END Import ${DATABASE} ${DATE} --  

exit 1
